package sun.misc;

import java.io.FileInputStream;
import java.net.URI;
import java.util.Arrays;
import java.util.jar.Attributes;
import java.util.Set;
import java.util.HashSet;
import java.util.zip.ZipEntry;
import java.security.CodeSigner;
import java.security.cert.Certificate;
import java.util.jar.Manifest;
import java.security.AccessControlException;
import java.util.jar.JarEntry;
import java.io.InputStream;
import java.io.FileNotFoundException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.util.jar.JarFile;
import java.io.Closeable;
import sun.security.action.GetPropertyAction;
import java.security.Permission;
import java.net.URLConnection;
import java.net.JarURLConnection;
import java.net.SocketPermission;
import java.io.FilePermission;
import sun.net.www.ParseUtil;
import java.util.StringTokenizer;
import java.io.File;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.security.PrivilegedExceptionAction;
import sun.net.util.URLUtil;
import java.util.NoSuchElementException;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Collections;
import java.util.LinkedList;
import java.io.IOException;
import java.util.List;
import java.net.URLStreamHandlerFactory;
import java.security.AccessControlContext;
import java.net.URLStreamHandler;
import java.util.HashMap;
import java.util.Stack;
import java.net.URL;
import java.util.ArrayList;

public class URLClassPath
{
    static final String USER_AGENT_JAVA_VERSION = "UA-Java-Version";
    static final String JAVA_VERSION;
    private static final boolean DEBUG;
    private static final boolean DEBUG_LOOKUP_CACHE;
    private static final boolean DISABLE_JAR_CHECKING;
    private static final boolean DISABLE_ACC_CHECKING;
    private static final boolean DISABLE_CP_URL_CHECK;
    private static final boolean DEBUG_CP_URL_CHECK;
    private ArrayList<URL> path;
    Stack<URL> urls;
    ArrayList<URLClassPath.Loader> loaders;
    HashMap<String, URLClassPath.Loader> lmap;
    private URLStreamHandler jarHandler;
    private boolean closed;
    private final AccessControlContext acc;
    private static volatile boolean lookupCacheEnabled;
    private URL[] lookupCacheURLs;
    private ClassLoader lookupCacheLoader;
    
    public URLClassPath(URL[] array, URLStreamHandlerFactory urlStreamHandlerFactory, AccessControlContext acc) {
        this.path = new ArrayList();
        this.urls = new Stack();
        this.loaders = new ArrayList();
        this.lmap = new HashMap();
        this.closed = false;
        for (int i = 0; i < array.length; ++i) {
            this.path.add(array[i]);
        }
        this.push(array);
        if (urlStreamHandlerFactory != null) {
            this.jarHandler = urlStreamHandlerFactory.createURLStreamHandler("jar");
        }
        if (URLClassPath.DISABLE_ACC_CHECKING) {
            this.acc = null;
        }
        else {
            this.acc = acc;
        }
    }
    
    public URLClassPath(URL[] array) {
        this(array, null, null);
    }
    
    public URLClassPath(URL[] array, AccessControlContext accessControlContext) {
        this(array, null, accessControlContext);
    }
    
    public synchronized List<IOException> closeLoaders() {
        if (this.closed) {
            return Collections.emptyList();
        }
        LinkedList<IOException> list = new LinkedList();
        for (URLClassPath.Loader loader : this.loaders) {
            try {
                loader.close();
            }
            catch (IOException ex) {
                list.add(ex);
            }
        }
        this.closed = true;
        return list;
    }
    
    public synchronized void addURL(URL e) {
        if (this.closed) {
            return;
        }
        synchronized (this.urls) {
            if (e == null || this.path.contains(e)) {
                return;
            }
            this.urls.add(0, e);
            this.path.add(e);
            if (this.lookupCacheURLs != null) {
                disableAllLookupCaches();
            }
        }
        return;
    }
    
    public URL[] getURLs() {
        synchronized (this.urls) {
            return (URL[])this.path.toArray(new URL[this.path.size()]);
        }
    }
    
    public URL findResource(String s, boolean b) {
        int[] lookupCache = this.getLookupCache(s);
        int n = 0;
        URLClassPath.Loader nextLoader;
        while ((nextLoader = this.getNextLoader(lookupCache, n)) != null) {
            URL resource = nextLoader.findResource(s, b);
            if (resource != null) {
                return resource;
            }
            ++n;
        }
        return null;
    }
    
    public Resource getResource(String str, boolean b) {
        if (URLClassPath.DEBUG) {
            System.err.println("URLClassPath.getResource(\"" + str + "\")");
        }
        int[] lookupCache = this.getLookupCache(str);
        int n = 0;
        URLClassPath.Loader nextLoader;
        while ((nextLoader = this.getNextLoader(lookupCache, n)) != null) {
            Resource resource = nextLoader.getResource(str, b);
            if (resource != null) {
                return resource;
            }
            ++n;
        }
        return null;
    }
    
    public Enumeration<URL> findResources(final String s, final boolean b) {
        return new Enumeration<URL>() {
            private int index = 0;
            private int[] cache = URLClassPath.access$000(URLClassPath.this, s);
            private URL url = null;
            
            private boolean next() {
                if (this.url != null) {
                    return true;
                }
                URLClassPath.Loader access$100;
                while ((access$100 = URLClassPath.access$100(URLClassPath.this, this.cache, this.index++)) != null) {
                    this.url = access$100.findResource(s, b);
                    if (this.url == null) {
                        continue;
                    }
                    return true;
                }
                return false;
            }
            
            @Override
            public boolean hasMoreElements() {
                return this.next();
            }
            
            @Override
            public URL nextElement() {
                if (!this.next()) {
                    throw new NoSuchElementException();
                }
                URL url = this.url;
                this.url = null;
                return url;
            }
        };
    }
    
    public Resource getResource(String s) {
        return this.getResource(s, true);
    }
    
    public Enumeration<Resource> getResources(final String s, final boolean b) {
        return new Enumeration<Resource>() {
            private int index = 0;
            private int[] cache = URLClassPath.access$000(URLClassPath.this, s);
            private Resource res = null;
            
            private boolean next() {
                if (this.res != null) {
                    return true;
                }
                URLClassPath.Loader access$100;
                while ((access$100 = URLClassPath.access$100(URLClassPath.this, this.cache, this.index++)) != null) {
                    this.res = access$100.getResource(s, b);
                    if (this.res == null) {
                        continue;
                    }
                    return true;
                }
                return false;
            }
            
            @Override
            public boolean hasMoreElements() {
                return this.next();
            }
            
            @Override
            public Resource nextElement() {
                if (!this.next()) {
                    throw new NoSuchElementException();
                }
                Resource res = this.res;
                this.res = null;
                return res;
            }
        };
    }
    
    public Enumeration<Resource> getResources(String s) {
        return this.getResources(s, true);
    }
    
    synchronized void initLookupCache(ClassLoader lookupCacheLoader) {
        URL[] lookupCacheURLs = getLookupCacheURLs(lookupCacheLoader);
        this.lookupCacheURLs = lookupCacheURLs;
        if (lookupCacheURLs != null) {
            this.lookupCacheLoader = lookupCacheLoader;
        }
        else {
            disableAllLookupCaches();
        }
    }
    
    static void disableAllLookupCaches() {
        URLClassPath.lookupCacheEnabled = false;
    }
    
    private static native URL[] getLookupCacheURLs(ClassLoader p0);
    
    private static native int[] getLookupCacheForClassLoader(ClassLoader p0, String p1);
    
    private static native boolean knownToNotExist0(ClassLoader p0, String p1);
    
    synchronized boolean knownToNotExist(String s) {
        return this.lookupCacheURLs != null && URLClassPath.lookupCacheEnabled && knownToNotExist0(this.lookupCacheLoader, s);
    }
    
    private synchronized int[] getLookupCache(String s) {
        if (this.lookupCacheURLs == null || !URLClassPath.lookupCacheEnabled) {
            return null;
        }
        int[] lookupCacheForClassLoader = getLookupCacheForClassLoader(this.lookupCacheLoader, s);
        if (lookupCacheForClassLoader != null && lookupCacheForClassLoader.length > 0) {
            int i = lookupCacheForClassLoader[lookupCacheForClassLoader.length - 1];
            if (!this.ensureLoaderOpened(i)) {
                if (URLClassPath.DEBUG_LOOKUP_CACHE) {
                    System.out.println("Expanded loaders FAILED " + this.loaders.size() + " for maxindex=" + i);
                }
                return null;
            }
        }
        return lookupCacheForClassLoader;
    }
    
    private boolean ensureLoaderOpened(int i) {
        if (this.loaders.size() <= i) {
            if (this.getLoader(i) == null) {
                return false;
            }
            if (!URLClassPath.lookupCacheEnabled) {
                return false;
            }
            if (URLClassPath.DEBUG_LOOKUP_CACHE) {
                System.out.println("Expanded loaders " + this.loaders.size() + " to index=" + i);
            }
        }
        return true;
    }
    
    private synchronized void validateLookupCache(int i, String s) {
        if (this.lookupCacheURLs != null && URLClassPath.lookupCacheEnabled) {
            if (i < this.lookupCacheURLs.length && s.equals(URLUtil.urlNoFragString(this.lookupCacheURLs[i]))) {
                return;
            }
            if (URLClassPath.DEBUG || URLClassPath.DEBUG_LOOKUP_CACHE) {
                System.out.println("WARNING: resource lookup cache invalidated for lookupCacheLoader at " + i);
            }
            disableAllLookupCaches();
        }
    }
    
    private synchronized URLClassPath.Loader getNextLoader(int[] array, int n) {
        if (this.closed) {
            return null;
        }
        if (array != null) {
            if (n < array.length) {
                URLClassPath.Loader loader = (URLClassPath.Loader)this.loaders.get(array[n]);
                if (URLClassPath.DEBUG_LOOKUP_CACHE) {
                    System.out.println("HASCACHE: Loading from : " + array[n] + " = " + loader.getBaseURL());
                    return loader;
                }
                return loader;
            }
            else {
                return null;
            }
        }
        else {
            return this.getLoader(n);
        }
    }
    
    private synchronized URLClassPath.Loader getLoader(int n) {
        if (this.closed) {
            return null;
        }
        while (this.loaders.size() < n + 1) {
            URL url;
            synchronized (this.urls) {
                if (this.urls.empty()) {
                    return null;
                }
                url = (URL)this.urls.pop();
            }
            String urlNoFragString = URLUtil.urlNoFragString(url);
            if (this.lmap.containsKey(urlNoFragString)) {
                continue;
            }
            URLClassPath.Loader loader2;
            try {
                loader2 = this.getLoader(url);
                URL[] classPath = loader2.getClassPath();
                if (classPath != null) {
                    this.push(classPath);
                }
            }
            catch (IOException ex) {
                continue;
            }
            catch (SecurityException obj) {
                if (URLClassPath.DEBUG) {
                    System.err.println("Failed to access " + url + ", " + obj);
                }
                else {
                    continue;
                }
                continue;
            }
            this.validateLookupCache(this.loaders.size(), urlNoFragString);
            this.loaders.add(loader2);
            this.lmap.put(urlNoFragString, loader2);
        }
        if (URLClassPath.DEBUG_LOOKUP_CACHE) {
            System.out.println("NOCACHE: Loading from : " + n);
        }
        return (URLClassPath.Loader)this.loaders.get(n);
    }
    
    private URLClassPath.Loader getLoader(final URL url) throws IOException {
        try {
            return (URLClassPath.Loader)AccessController.doPrivileged(new PrivilegedExceptionAction<URLClassPath.Loader>() {
                @Override
                public URLClassPath.Loader run() throws IOException {
                    String file = url.getFile();
                    if (file != null && file.endsWith("/")) {
                        if ("file".equals(url.getProtocol())) {
                            return new FileLoader(url);
                        }
                        return new Loader(url);
                    }
                    else {
                        return new URLClassPath.JarLoader(url, URLClassPath.this.jarHandler, URLClassPath.this.lmap, URLClassPath.this.acc);
                    }
                }
            }, this.acc);
        }
        catch (PrivilegedActionException ex) {
            throw (IOException)ex.getException();
        }
    }
    
    private void push(URL[] array) {
        synchronized (this.urls) {
            for (int i = array.length - 1; i >= 0; --i) {
                this.urls.push(array[i]);
            }
        }
    }
    
    public static URL[] pathToURLs(String str) {
        StringTokenizer stringTokenizer = new StringTokenizer(str, File.pathSeparator);
        URL[] array = new URL[stringTokenizer.countTokens()];
        int n = 0;
        while (stringTokenizer.hasMoreTokens()) {
            File file = new File(stringTokenizer.nextToken());
            try {
                file = new File(file.getCanonicalPath());
            }
            catch (IOException ex) {}
            try {
                array[n++] = ParseUtil.fileToEncodedURL(file);
            }
            catch (IOException ex2) {}
        }
        if (array.length != n) {
            URL[] array2 = new URL[n];
            System.arraycopy(array, 0, array2, 0, n);
            array = array2;
            return array;
        }
        return array;
    }
    
    public URL checkURL(URL url) {
        try {
            check(url);
        }
        catch (Exception ex) {
            return null;
        }
        return url;
    }
    
    static void check(URL jarFileURL) throws IOException {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            URLConnection openConnection = jarFileURL.openConnection();
            Permission permission = openConnection.getPermission();
            if (permission != null) {
                try {
                    securityManager.checkPermission(permission);
                }
                catch (SecurityException ex) {
                    if (permission instanceof FilePermission && permission.getActions().indexOf("read") != -1) {
                        securityManager.checkRead(permission.getName());
                    }
                    else if (permission instanceof SocketPermission && permission.getActions().indexOf("connect") != -1) {
                        if ((openConnection instanceof JarURLConnection)) {
                            jarFileURL = ((JarURLConnection)openConnection).getJarFileURL();
                        }
                        securityManager.checkConnect(jarFileURL.getHost(), jarFileURL.getPort());
                    }
                    else {
                        throw ex;
                    }
                }
            }
        }
    }
    
    static int[] access$000(URLClassPath urlClassPath, String s) {
        return urlClassPath.getLookupCache(s);
    }
    
    static URLClassPath.Loader access$100(URLClassPath urlClassPath, int[] array, int n) {
        return urlClassPath.getNextLoader(array, n);
    }
    
    static {
        JAVA_VERSION = (String)AccessController.doPrivileged(new GetPropertyAction("java.version"));
        DEBUG = (AccessController.doPrivileged(new GetPropertyAction("sun.misc.URLClassPath.debug")) != null);
        DEBUG_LOOKUP_CACHE = (AccessController.doPrivileged(new GetPropertyAction("sun.misc.URLClassPath.debugLookupCache")) != null);
        String s = (String)AccessController.doPrivileged(new GetPropertyAction("sun.misc.URLClassPath.disableJarChecking"));
        DISABLE_JAR_CHECKING = (s != null && (s.equals("true") || s.equals("")));
        String s2 = (String)AccessController.doPrivileged(new GetPropertyAction("jdk.net.URLClassPath.disableRestrictedPermissions"));
        DISABLE_ACC_CHECKING = (s2 != null && (s2.equals("true") || s2.equals("")));
        String anObject = (String)AccessController.doPrivileged(new GetPropertyAction("jdk.net.URLClassPath.disableClassPathURLCheck", "true"));
        DISABLE_CP_URL_CHECK = (anObject != null && (anObject.equals("true") || anObject.isEmpty()));
        DEBUG_CP_URL_CHECK = "debug".equals(anObject);
        URLClassPath.lookupCacheEnabled = "true".equals(VM.getSavedProperty("sun.cds.enableSharedLookupCache"));
    }
    
    private static class Loader implements Closeable
    {
        private final URL base;
        private JarFile jarfile;
        
        Loader(URL base) {
            this.base = base;
        }
        
        URL getBaseURL() {
            return this.base;
        }
        
        URL findResource(String path, boolean b) {
            URL url;
            try {
                url = new URL(this.base, ParseUtil.encodePath(path, false));
            }
            catch (MalformedURLException ex) {
                throw new IllegalArgumentException("name");
            }
            try {
                if (b) {
                    URLClassPath.check(url);
                }
                URLConnection openConnection = url.openConnection();
                if ((openConnection instanceof HttpURLConnection)) {
                    HttpURLConnection httpURLConnection = (HttpURLConnection)openConnection;
                    httpURLConnection.setRequestMethod("HEAD");
                    if (httpURLConnection.getResponseCode() >= 400) {
                        return null;
                    }
                }
                else {
                    openConnection.setUseCaches(false);
                    openConnection.getInputStream().close();
                }
                return url;
            }
            catch (Exception ex2) {
                return null;
            }
        }
        
        Resource getResource(String path, boolean b) {
            URL url;
            try {
                url = new URL(this.base, ParseUtil.encodePath(path, false));
            }
            catch (MalformedURLException ex) {
                throw new IllegalArgumentException("name");
            }
            URLConnection openConnection;
            try {
                if (b) {
                    URLClassPath.check(url);
                }
                openConnection = url.openConnection();
                if ((openConnection instanceof JarURLConnection)) {
                    JarURLConnection jarURLConnection = (JarURLConnection)openConnection;
                    try {
                        JarFile checkJar = URLClassPath.JarLoader.checkJar(jarURLConnection.getJarFile());
                        if (jarURLConnection.getUseCaches()) {
                            this.jarfile = checkJar;
                        }
                    }
                    catch (FileNotFoundException ex2) {
                        if (jarURLConnection.getUseCaches()) {
                            this.jarfile = URLClassPath.JarLoader.checkJar(((JarURLConnection)new URL("jar:" + jarURLConnection.getJarFileURL() + "!/").openConnection()).getJarFile());
                        }
                        throw ex2;
                    }
                }
                openConnection.getInputStream();
            }
            catch (Exception ex3) {
                return null;
            }
            return new Resource() {
                @Override
                public String getName() {
                    return path;
                }
                
                @Override
                public URL getURL() {
                    return url;
                }
                
                @Override
                public URL getCodeSourceURL() {
                    return URLClassPath.Loader.this.base;
                }
                
                @Override
                public InputStream getInputStream() throws IOException {
                    return openConnection.getInputStream();
                }
                
                @Override
                public int getContentLength() throws IOException {
                    return openConnection.getContentLength();
                }
            };
        }
        
        Resource getResource(String s) {
            return this.getResource(s, true);
        }
        
        @Override
        public void close() throws IOException {
            if (this.jarfile != null) {
                this.jarfile.close();
            }
        }
        
        URL[] getClassPath() throws IOException {
            return null;
        }
    }
    
    static class JarLoader extends URLClassPath.Loader
    {
        private JarFile jar;
        private final URL csu;
        private JarIndex index;
        private MetaIndex metaIndex;
        private URLStreamHandler handler;
        private final HashMap<String, URLClassPath.Loader> lmap;
        private final AccessControlContext acc;
        private boolean closed;
        private static final JavaUtilZipFileAccess zipAccess;
        
        JarLoader(URL url, URLStreamHandler urlStreamHandler, HashMap<String, URLClassPath.Loader> lmap, AccessControlContext acc) throws IOException {
            super(new URL("jar", "", -1, url + "!/", urlStreamHandler));
            this.closed = false;
            this.csu = url;
            this.handler = urlStreamHandler;
            this.lmap = lmap;
            this.acc = acc;
            if (!this.isOptimizable(url)) {
                this.ensureOpen();
            }
            else {
                String file = url.getFile();
                if (file != null) {
                    File file2 = new File(ParseUtil.decode(file));
                    this.metaIndex = MetaIndex.forJar(file2);
                    if (this.metaIndex != null && !file2.exists()) {
                        this.metaIndex = null;
                    }
                }
                if (this.metaIndex == null) {
                    this.ensureOpen();
                }
            }
        }
        
        @Override
        public void close() throws IOException {
            if (!this.closed) {
                this.closed = true;
                this.ensureOpen();
                this.jar.close();
            }
        }
        
        JarFile getJarFile() {
            return this.jar;
        }
        
        private boolean isOptimizable(URL url) {
            return "file".equals(url.getProtocol());
        }
        
        private void ensureOpen() throws IOException {
            if (this.jar == null) {
                try {
                    AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
                        @Override
                        public Void run() throws IOException {
                            if (URLClassPath.DEBUG) {
                                System.err.println("Opening " + URLClassPath.JarLoader.this.csu);
                                Thread.dumpStack();
                            }
                            URLClassPath.JarLoader.this.jar = URLClassPath.JarLoader.access$800(URLClassPath.JarLoader.this, URLClassPath.JarLoader.this.csu);
                            URLClassPath.JarLoader.this.index = JarIndex.getJarIndex(URLClassPath.JarLoader.this.jar, URLClassPath.JarLoader.this.metaIndex);
                            if (URLClassPath.JarLoader.this.index != null) {
                                String[] jarFiles = URLClassPath.JarLoader.this.index.getJarFiles();
                                for (int i = 0; i < jarFiles.length; ++i) {
                                    try {
                                        String urlNoFragString = URLUtil.urlNoFragString(new URL(URLClassPath.JarLoader.this.csu, jarFiles[i]));
                                        if (!URLClassPath.JarLoader.this.lmap.containsKey(urlNoFragString)) {
                                            URLClassPath.JarLoader.this.lmap.put(urlNoFragString, null);
                                        }
                                    }
                                    catch (MalformedURLException ex) {}
                                }
                            }
                            return null;
                        }
                    }, this.acc);
                }
                catch (PrivilegedActionException ex) {
                    throw (IOException)ex.getException();
                }
            }
        }
        
        static JarFile checkJar(JarFile jarFile) throws IOException {
            if (System.getSecurityManager() != null && !URLClassPath.DISABLE_JAR_CHECKING && !URLClassPath.JarLoader.zipAccess.startsWithLocHeader(jarFile)) {
                IOException ex = new IOException("Invalid Jar file");
                try {
                    jarFile.close();
                }
                catch (IOException exception) {
                    ex.addSuppressed(exception);
                }
                throw ex;
            }
            return jarFile;
        }
        
        private JarFile getJarFile(URL url) throws IOException {
            if (this.isOptimizable(url)) {
                FileURLMapper fileURLMapper = new FileURLMapper(url);
                if (!fileURLMapper.exists()) {
                    throw new FileNotFoundException(fileURLMapper.getPath());
                }
                return checkJar(new JarFile(fileURLMapper.getPath()));
            }
            else {
                URLConnection openConnection = this.getBaseURL().openConnection();
                openConnection.setRequestProperty("UA-Java-Version", URLClassPath.JAVA_VERSION);
                return checkJar(((JarURLConnection)openConnection).getJarFile());
            }
        }
        
        JarIndex getIndex() {
            try {
                this.ensureOpen();
            }
            catch (IOException cause) {
                throw new InternalError(cause);
            }
            return this.index;
        }
        
        Resource checkResource(String path, boolean b, JarEntry jarEntry) {
            URL url;
            try {
                url = new URL(this.getBaseURL(), ParseUtil.encodePath(path, false));
                if (b) {
                    URLClassPath.check(url);
                }
            }
            catch (MalformedURLException ex) {
                return null;
            }
            catch (IOException ex2) {
                return null;
            }
            catch (AccessControlException ex3) {
                return null;
            }
            return new Resource() {
                @Override
                public String getName() {
                    return path;
                }
                
                @Override
                public URL getURL() {
                    return url;
                }
                
                @Override
                public URL getCodeSourceURL() {
                    return URLClassPath.JarLoader.this.csu;
                }
                
                @Override
                public InputStream getInputStream() throws IOException {
                    return URLClassPath.JarLoader.this.jar.getInputStream(jarEntry);
                }
                
                @Override
                public int getContentLength() {
                    return (int)jarEntry.getSize();
                }
                
                @Override
                public Manifest getManifest() throws IOException {
                    SharedSecrets.javaUtilJarAccess().ensureInitialization(URLClassPath.JarLoader.this.jar);
                    return URLClassPath.JarLoader.this.jar.getManifest();
                }
                
                @Override
                public Certificate[] getCertificates() {
                    return jarEntry.getCertificates();
                }
                
                @Override
                public CodeSigner[] getCodeSigners() {
                    return jarEntry.getCodeSigners();
                }
            };
        }
        
        boolean validIndex(String substring) {
            int lastIndex;
            if ((lastIndex = substring.lastIndexOf("/")) != -1) {
                substring = substring.substring(0, lastIndex);
            }
            Enumeration<JarEntry> entries = this.jar.entries();
            while (entries.hasMoreElements()) {
                String s = ((ZipEntry)entries.nextElement()).getName();
                int lastIndex2;
                if ((lastIndex2 = s.lastIndexOf("/")) != -1) {
                    s = s.substring(0, lastIndex2);
                }
                if (!s.equals(substring)) {
                    continue;
                }
                return true;
            }
            return false;
        }
        
        @Override
        URL findResource(String s, boolean b) {
            Resource resource = this.getResource(s, b);
            if (resource != null) {
                return resource.getURL();
            }
            return null;
        }
        
        @Override
        Resource getResource(String name, boolean b) {
            if (this.metaIndex != null && !this.metaIndex.mayContain(name)) {
                return null;
            }
            try {
                this.ensureOpen();
            }
            catch (IOException cause) {
                throw new InternalError(cause);
            }
            JarEntry jarEntry = this.jar.getJarEntry(name);
            if (jarEntry != null) {
                return this.checkResource(name, b, jarEntry);
            }
            if (this.index == null) {
                return null;
            }
            return this.getResource(name, b, new HashSet());
        }
        
        Resource getResource(String name, boolean b, Set<String> set) {
            int i = 0;
            LinkedList<String> list;
            if ((list = this.index.get(name)) == null) {
                return null;
            }
            do {
                int size = list.size();
                String[] array = (String[])list.toArray(new String[size]);
                while (i < size) {
                    String spec = array[i++];
                    Object o;
                    URLClassPath.JarLoader value;
                    try {
                        o = new URL(this.csu, spec);
                        String urlNoFragString = URLUtil.urlNoFragString(o);
                        if ((value = (URLClassPath.JarLoader)this.lmap.get(urlNoFragString)) == null) {
                            value = (URLClassPath.JarLoader)AccessController.doPrivileged(new PrivilegedExceptionAction<URLClassPath.JarLoader>() {
                                @Override
                                public URLClassPath.JarLoader run() throws IOException {
                                    return new URLClassPath.JarLoader(o, URLClassPath.JarLoader.this.handler, URLClassPath.JarLoader.this.lmap, URLClassPath.JarLoader.this.acc);
                                }
                            }, this.acc);
                            JarIndex index = value.getIndex();
                            if (index != null) {
                                int lastIndex = spec.lastIndexOf("/");
                                index.merge(this.index, (lastIndex == -1) ? null : spec.substring(0, lastIndex + 1));
                            }
                            this.lmap.put(urlNoFragString, value);
                        }
                    }
                    catch (PrivilegedActionException ex) {
                        continue;
                    }
                    catch (MalformedURLException ex2) {
                        continue;
                    }
                    int n = !set.add(URLUtil.urlNoFragString(o));
                    if (n == 0) {
                        try {
                            value.ensureOpen();
                        }
                        catch (IOException cause) {
                            throw new InternalError(cause);
                        }
                        JarEntry jarEntry = value.jar.getJarEntry(name);
                        if (jarEntry != null) {
                            return value.checkResource(name, b, jarEntry);
                        }
                        if (!value.validIndex(name)) {
                            throw new InvalidJarIndexException("Invalid index");
                        }
                    }
                    if (n || value == this) {
                        continue;
                    }
                    if (value.getIndex() == null) {
                        continue;
                    }
                    Resource resource;
                    if ((resource = value.getResource(name, b, set)) != null) {
                        return resource;
                    }
                    continue;
                }
                list = this.index.get(name);
            } while (i < list.size());
            return null;
        }
        
        @Override
        URL[] getClassPath() throws IOException {
            if (this.index != null) {
                return null;
            }
            if (this.metaIndex != null) {
                return null;
            }
            this.ensureOpen();
            this.parseExtensionsDependencies();
            if (SharedSecrets.javaUtilJarAccess().jarFileHasClassPathAttribute(this.jar)) {
                Manifest manifest = this.jar.getManifest();
                if (manifest != null) {
                    Attributes mainAttributes = manifest.getMainAttributes();
                    if (mainAttributes != null) {
                        String value = mainAttributes.getValue(Attributes.Name.CLASS_PATH);
                        if (value != null) {
                            return this.parseClassPath(this.csu, value);
                        }
                    }
                }
            }
            return null;
        }
        
        private void parseExtensionsDependencies() throws IOException {
            ExtensionDependency.checkExtensionsDependencies(this.jar);
        }
        
        private URL[] parseClassPath(URL url, String str) throws MalformedURLException {
            StringTokenizer stringTokenizer = new StringTokenizer(str);
            URL[] original = new URL[stringTokenizer.countTokens()];
            int newLength = 0;
            while (stringTokenizer.hasMoreTokens()) {
                String nextToken = stringTokenizer.nextToken();
                Object o = URLClassPath.DISABLE_CP_URL_CHECK ? new URL(url, nextToken) : tryResolve(url, nextToken);
                if (o != null) {
                    original[newLength] = o;
                    ++newLength;
                }
                else if (URLClassPath.DEBUG_CP_URL_CHECK) {
                    System.err.println("Class-Path entry: \"" + nextToken + "\" ignored in JAR file " + url);
                }
                else {
                    continue;
                }
            }
            if (newLength == 0) {
                original = null;
            }
            else if (newLength != original.length) {
                original = (URL[])Arrays.copyOf(original, newLength);
            }
            return original;
        }
        
        static URL tryResolve(URL url, String s) throws MalformedURLException {
            if ("file".equalsIgnoreCase(url.getProtocol())) {
                return tryResolveFile(url, s);
            }
            return tryResolveNonFile(url, s);
        }
        
        static URL tryResolveFile(URL context, String spec) throws MalformedURLException {
            int index = spec.indexOf(58);
            return (index < 0 || "file".equalsIgnoreCase(spec.substring(0, index))) ? new URL(context, spec) : null;
        }
        
        static URL tryResolveNonFile(URL context, String s) throws MalformedURLException {
            String replace = s.replace(File.separatorChar, '/');
            if (isRelative(replace)) {
                URL url = new URL(context, replace);
                String path = context.getPath();
                String path2 = url.getPath();
                int lastIndex = path.lastIndexOf(47);
                if (lastIndex == -1) {
                    lastIndex = path.length() - 1;
                }
                if (path2.regionMatches(0, path, 0, lastIndex + 1) && path2.indexOf("..", lastIndex) == -1) {
                    return url;
                }
            }
            return null;
        }
        
        static boolean isRelative(String str) {
            try {
                return !URI.create(str).isAbsolute();
            }
            catch (IllegalArgumentException ex) {
                return false;
            }
        }
        
        static JarFile access$800(URLClassPath.JarLoader jarLoader, URL url) throws IOException {
            return jarLoader.getJarFile(url);
        }
        
        static {
            zipAccess = SharedSecrets.getJavaUtilZipFileAccess();
        }
    }
    
    private static class FileLoader extends URLClassPath.Loader
    {
        private File dir;
        
        FileLoader(URL url) throws IOException {
            super(url);
            if (!"file".equals(url.getProtocol())) {
                throw new IllegalArgumentException("url");
            }
            this.dir = new File(ParseUtil.decode(url.getFile().replace('/', File.separatorChar))).getCanonicalFile();
            return;
        }
        
        @Override
        URL findResource(String s, boolean b) {
            Resource resource = this.getResource(s, b);
            if (resource != null) {
                return resource.getURL();
            }
            return null;
        }
        
        @Override
        Resource getResource(String path, boolean b) {
            try {
                URL url = new URL(this.getBaseURL(), ".");
                URL url2 = new URL(this.getBaseURL(), ParseUtil.encodePath(path, false));
                if (!url2.getFile().startsWith(url.getFile())) {
                    return null;
                }
                if (b) {
                    URLClassPath.check(url2);
                }
                File canonicalFile;
                if (path.indexOf("..") != -1) {
                    canonicalFile = new File(this.dir, path.replace('/', File.separatorChar)).getCanonicalFile();
                    if (!canonicalFile.getPath().startsWith(this.dir.getPath())) {
                        return null;
                    }
                }
                else {
                    canonicalFile = new File(this.dir, path.replace('/', File.separatorChar));
                }
                if (canonicalFile.exists()) {
                    return new Resource() {
                        @Override
                        public String getName() {
                            return path;
                        }
                        
                        @Override
                        public URL getURL() {
                            return url2;
                        }
                        
                        @Override
                        public URL getCodeSourceURL() {
                            return URLClassPath.FileLoader.this.getBaseURL();
                        }
                        
                        @Override
                        public InputStream getInputStream() throws IOException {
                            return new FileInputStream(canonicalFile);
                        }
                        
                        @Override
                        public int getContentLength() throws IOException {
                            return (int)canonicalFile.length();
                        }
                    };
                }
            }
            catch (Exception ex) {
                return null;
            }
            return null;
        }
    }
}
